@@ -1,20 +1,21 @@
 /*
  * ml.c
  *
- * (c) 2008-2020  Klaus Schliep (klaus.schliep@gmail.com)
+ * (c) 2008-2021  Klaus Schliep (klaus.schliep@gmail.com)
  *
  *
  * This code may be distributed under the GNU GPL
  *
  */
-
-# define USE_RINTERNALS
+#define USE_FC_LEN_T
 
 #include <Rmath.h>
-//#include <math.h>
 #include <R.h>
-#include <R_ext/Lapack.h>
 #include <Rinternals.h>
+#include <R_ext/BLAS.h>
+#ifndef FCONE
+# define FCONE
+#endif
 
 
 #define LINDEX(i, k) (i - ntips - 1L) * (nr * nc) + k * ntips * (nr * nc)
@@ -23,14 +24,13 @@
 // index for scaling matrix SCM
 #define LINDEX3(i, j) (i - *ntips - 1L) * *nr + j * *ntips * *nr  //nr statt *nr
 
-char *transa = "N", *transb = "N";
+
 double one = 1.0, zero = 0.0;
 int ONE = 1L;
 const double ScaleEPS = 1.0/4294967296.0;
 const double ScaleMAX = 4294967296.0;
 const double LOG_SCALE_EPS = -22.18070977791824915926;
 
-// 2^64 = 18446744073709551616
 
 static double *LL;  //, *WEIGHTS;
 static int *SCM; //, *XXX;
@@ -56,25 +56,9 @@ SEXP ll_init2(SEXP nr, SEXP nTips, SEXP nc, SEXP k)
 /*
 LL likelihood for internal edges
 SCM scaling coefficients
-nNodes, nTips, kmax
-
-void ll_init(int *nr, int *nTips, int *nc, int *k)
-{
-    int i;
-    LL = (double *) calloc(*nr * *nc * *k * *nTips, sizeof(double));
-    SCM = (int *) calloc(*nr * *k * *nTips, sizeof(int));  // * 2L
-    for(i =0; i < (*nr * *k * *nTips); i++) SCM[i] = 0L;
-}
-
-
-void ll_free(){
-    free(LL);
-    free(SCM);
-}
 */
 
 
-// in C++ ??
 void matm(int *x, double *contrast, int *nr, int *nc, int *nco, double *result){
     int i, j;
     for(i = 0; i < (*nr); i++){
@@ -113,12 +97,11 @@ void scaleMatrix(double *X, int *nr, int *nc, int *result){
 
 
 // contrast to full dense matrix
-// zwei Versionen *= und new
 void matp(int *x, double *contrast, double *P, int *nr, int *nc, int *nrs, double *result){
     int i, j;
     double *tmp;
     tmp = (double *) R_alloc((*nc) *(*nrs), sizeof(double));
-    F77_CALL(dgemm)(transa, transb, nrs, nc, nc, &one, contrast, nrs, P, nc, &zero, tmp, nrs);
+    F77_CALL(dgemm)("N", "N", nrs, nc, nc, &one, contrast, nrs, P, nc, &zero, tmp, nrs FCONE FCONE);
     for(i = 0; i < (*nr); i++){
         for(j = 0; j < (*nc); j++) result[i + j*(*nr)] = tmp[x[i] - 1L + j*(*nrs)];
     }
@@ -136,16 +119,13 @@ void rowMinScale(int *dat, int n,  int k, int *res){
     }
 }
 
-// Ziel etwas schneller
+
 void getP(double *eva, double *ev, double *evi, int m, double el, double w, double *result){
     int i, j, h;
-    double res; //tmp[m],
+    double res;
     double *tmp;
-//    tmp = (double *) malloc(m * sizeof(double));
     tmp = (double *) R_alloc(m, sizeof(double));
-// el = 0 return identity
     for(i = 0; i < m; i++) tmp[i] = exp(eva[i] * w * el);
-// eva *= tmp???
     for(i = 0; i < m; i++){
         for(j = 0; j < m; j++){
             res = 0.0;
@@ -153,9 +133,8 @@ void getP(double *eva, double *ev, double *evi, int m, double el, double w, doub
             result[i+j*m] = res;
         }
     }
-//    free(tmp);  // ausserhalb
 }
-// 64 * 3 + 4     16 + 64 * 2
+
 
 SEXP getPM(SEXP eig, SEXP nc, SEXP el, SEXP w){
     R_len_t i, j, nel, nw, k;
@@ -206,18 +185,18 @@ void lll(SEXP dlist, double *eva, double *eve, double *evei, double *el, double
             if(ei < nTips)
                 matp(INTEGER(VECTOR_ELT(dlist, ei)), contrast, P, nr, nc, &nco, &ans[ni * rc]);
             else
-                F77_CALL(dgemm)(transa, transb, nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, &ans[ni * rc], nr);
+                F77_CALL(dgemm)("N", "N", nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, &ans[ni * rc], nr FCONE FCONE);
         }
         else {
             if(ei < nTips)
                 matp(INTEGER(VECTOR_ELT(dlist, ei)), contrast, P, nr, nc, &nco, rtmp);
             else
-                F77_CALL(dgemm)(transa, transb, nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, rtmp, nr);
+                F77_CALL(dgemm)("N", "N", nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, rtmp, nr FCONE FCONE);
             for(j=0; j < rc; j++) ans[ni * rc + j] *= rtmp[j];
         }
     }
     scaleMatrix(&ans[ni * rc], nr, nc, scaleTmp);
-    F77_CALL(dgemv)(transa, nr, nc, &one, &ans[ni * rc], nr, bf, &ONE, &zero, TMP, &ONE);
+    F77_CALL(dgemv)("N", nr, nc, &one, &ans[ni * rc], nr, bf, &ONE, &zero, TMP, &ONE FCONE);
 }
 
 
@@ -232,20 +211,17 @@ void lll3(SEXP dlist, double *eva, double *eve, double *evei, double *el, double
     P = (double *) R_alloc(*nc * *nc, sizeof(double));
     for(j=0; j < *nr; j++) scaleTmp[j] = 0L;
     for(i = 0; i < n; i++) {
-// entweder P matrix erstellen, openMP ??
-// temp Vectoren vermeiden
         getP(eva, eve, evei, *nc, el[i], g, P);
         ei = edge[i];
         if(ni != node[i]){
 // test for node[i+1]
-// temp Vectoren vermeiden
             if(ni>0)scaleMatrix(&ans[ni * rc], nr, nc, &SC[ni * *nr]); // (ni-nTips)
             ni = node[i];
             for(j=0; j < *nr; j++) SC[j + ni * *nr] = 0L;
             if(ei < nTips)
                 matp(INTEGER(VECTOR_ELT(dlist, ei)), contrast, P, nr, nc, &nco, &ans[ni * rc]);
             else{
-                F77_CALL(dgemm)(transa, transb, nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, &ans[ni * rc], nr);
+                F77_CALL(dgemm)("N", "N", nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, &ans[ni * rc], nr FCONE FCONE);
                 for(j=0; j < *nr; j++) SC[ni * *nr + j] = SC[(ei-nTips) * *nr + j];
             }
         }
@@ -253,7 +229,7 @@ void lll3(SEXP dlist, double *eva, double *eve, double *evei, double *el, double
             if(ei < nTips)
                 matp(INTEGER(VECTOR_ELT(dlist, ei)), contrast, P, nr, nc, &nco, rtmp);
             else{
-                F77_CALL(dgemm)(transa, transb, nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, rtmp, nr);
+                F77_CALL(dgemm)("N", "N", nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, rtmp, nr FCONE FCONE);
                 for(j=0; j < *nr; j++) SC[ni * *nr + j] += SC[(ei-nTips) * *nr + j];
             }
             for(j=0; j < rc; j++) ans[ni * rc + j] *= rtmp[j];
@@ -262,36 +238,10 @@ void lll3(SEXP dlist, double *eva, double *eve, double *evei, double *el, double
     scaleMatrix(&ans[ni * rc], nr, nc, &SC[ni * *nr]);
     for(j=0; j < *nr; j++) scaleTmp[j] = SC[ni * *nr + j];
 
-    F77_CALL(dgemv)(transa, nr, nc, &one, &ans[ni * rc], nr, bf, &ONE, &zero, TMP, &ONE);
+    F77_CALL(dgemv)("N", nr, nc, &one, &ans[ni * rc], nr, bf, &ONE, &zero, TMP, &ONE FCONE);
 }
 
 
-// in pml.move inside optimRooted  mit SCM
-// , SEXP W
-SEXP PML3(SEXP dlist, SEXP EL, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP nco, SEXP contrast, SEXP N){
-    int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL;
-    int nTips = INTEGER(NTips)[0], *SC;
-    double *g=REAL(G), *tmp, logScaleEPS;
-    SEXP TMP;
-    double *eva, *eve, *evei;
-    eva = REAL(VECTOR_ELT(eig, 0));
-    eve = REAL(VECTOR_ELT(eig, 1));
-    evei = REAL(VECTOR_ELT(eig, 2));
-    SC = (int *) R_alloc(nr * k, sizeof(int));
-    PROTECT(TMP = allocMatrix(REALSXP, nr, k)); // changed
-    tmp=REAL(TMP);
-    for(i=0; i<(k*nr); i++)tmp[i]=0.0;
-    indLL = nr * nc * nTips;
-    for(i=0; i<k; i++){
-        lll3(dlist, eva, eve, evei, REAL(EL), g[i], &nr, &nc, INTEGER(node), INTEGER(edge), nTips, REAL(contrast), INTEGER(nco)[0], INTEGER(N)[0],  &SC[nr * i], REAL(bf), &tmp[i*nr], &LL[indLL *i], &SCM[nr * nTips * i]);
-     }
-    logScaleEPS = log(ScaleEPS);
-    for(i=0; i<(k*nr); i++) tmp[i] = logScaleEPS * SC[i] + log(tmp[i]);
-    UNPROTECT(1);
-    return TMP;
-}
-
-// , SEXP W
 SEXP PML0(SEXP dlist, SEXP EL, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP nco, SEXP contrast, SEXP N){
     int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL;
     int nTips = INTEGER(NTips)[0], *SC;
@@ -317,26 +267,57 @@ SEXP PML0(SEXP dlist, SEXP EL, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP
 }
 
 
+SEXP PML4(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP nco, SEXP contrast, SEXP N){
+    int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, j, indLL;
+    int nTips = INTEGER(NTips)[0], *SC, *sc;
+    double *g=REAL(G), *w=REAL(W), *tmp, *res;
+    SEXP TMP;
+    double *eva, *eve, *evei;
+    eva = REAL(VECTOR_ELT(eig, 0));
+    eve = REAL(VECTOR_ELT(eig, 1));
+    evei = REAL(VECTOR_ELT(eig, 2));
+    SC = (int *) R_alloc(nr * k, sizeof(int));
+    sc = (int *) R_alloc(nr, sizeof(int));
+    tmp = (double *) R_alloc(nr * k, sizeof(double));
+    PROTECT(TMP = allocVector(REALSXP, nr));
+
+    res=REAL(TMP);
+    for(i=0; i<(k*nr); i++)tmp[i]=0.0;
+    indLL = nr * nc * nTips;
+    for(i=0; i<k; i++){
+        lll3(dlist, eva, eve, evei, REAL(EL), g[i], &nr, &nc, INTEGER(node), INTEGER(edge), nTips, REAL(contrast), INTEGER(nco)[0], INTEGER(N)[0],  &SC[nr * i], REAL(bf), &tmp[i*nr], &LL[indLL *i], &SCM[nr * nTips * i]);
+    }
+    rowMinScale(SC, nr, k, sc);
+    for(i=0; i<nr; i++){
+        res[i]=0.0;
+        for(j=0;j<k;j++)res[i] += w[j] * exp(LOG_SCALE_EPS * SC[i+j*nr]) * tmp[i+j*nr];
+    }
+    for(i=0; i<nr; i++) res[i] = log(res[i]) + LOG_SCALE_EPS * sc[i];
+    UNPROTECT(1);
+    return TMP;
+}
+
+
 void moveLL5(double *LL, double *child, double *P, int *nr, int *nc, double *tmp){
     int j;
-    F77_CALL(dgemm)(transa, transb, nr, nc, nc, &one, child, nr, P, nc, &zero, tmp, nr);
+    F77_CALL(dgemm)("N", "N", nr, nc, nc, &one, child, nr, P, nc, &zero, tmp, nr FCONE FCONE);
     for(j=0; j<(*nc * *nr); j++) LL[j]/=tmp[j];
-    F77_CALL(dgemm)(transa, transb, nr, nc, nc, &one, LL, nr, P, nc, &zero, tmp, nr);
+    F77_CALL(dgemm)("N", "N", nr, nc, nc, &one, LL, nr, P, nc, &zero, tmp, nr FCONE FCONE);
     for(j=0; j<(*nc * *nr); j++) child[j] *= tmp[j];
 }
 
 
 // dad / child * P
 void helpDADI(double *dad, double *child, double *P, int nr, int nc, double *res){
-    F77_CALL(dgemm)(transa, transb, &nr, &nc, &nc, &one, child, &nr, P, &nc, &zero, res, &nr);
+    F77_CALL(dgemm)("N", "N", &nr, &nc, &nc, &one, child, &nr, P, &nc, &zero, res, &nr FCONE FCONE);
     for(int j=0; j<(nc * nr); j++) dad[j]/=res[j];
 }
 
 
 // braucht Addition skalierte Werte
 void helpPrep(double *dad, double *child, double *eve, double *evi, int nr, int nc, double *tmp, double *res){
-    F77_CALL(dgemm)(transa, transb, &nr, &nc, &nc, &one, child, &nr, eve, &nc, &zero, res, &nr);
-    F77_CALL(dgemm)(transa, transb, &nr, &nc, &nc, &one, dad, &nr, evi, &nc, &zero, tmp, &nr);
+    F77_CALL(dgemm)("N", "N", &nr, &nc, &nc, &one, child, &nr, eve, &nc, &zero, res, &nr FCONE FCONE);
+    F77_CALL(dgemm)("N", "N", &nr, &nc, &nc, &one, dad, &nr, evi, &nc, &zero, tmp, &nr FCONE FCONE);
     for(int j=0; j<(nc * nr); j++) res[j]*=tmp[j];
 }
 
@@ -371,7 +352,7 @@ SEXP getDAD2(SEXP dad, SEXP child, SEXP contrast, SEXP P, SEXP nr, SEXP nc, SEXP
 
 void helpPrep2(double *dad, int *child, double *contrast, double *evi, int nr, int nc, int nrs, double *res){
     int i, j;
-    F77_CALL(dgemm)(transa, transb, &nr, &nc, &nc, &one, dad, &nr, evi, &nc, &zero, res, &nr);
+    F77_CALL(dgemm)("N", "N", &nr, &nc, &nc, &one, dad, &nr, evi, &nc, &zero, res, &nr FCONE FCONE);
     for(i = 0; i < nr; i++){
         for(j = 0; j < nc; j++) res[i + j*nr] *= contrast[child[i] - 1L + j*nrs];
     }
@@ -396,7 +377,7 @@ SEXP getPrep2(SEXP dad, SEXP child, SEXP contrast, SEXP evi, SEXP nr, SEXP nc, S
 
 // child *= (dad * P)
 void goDown(double *dad, double *child, double *P, int nr, int nc, double *res){
-    F77_CALL(dgemm)(transa, transb, &nr, &nc, &nc, &one, dad, &nr, P, &nc, &zero, res, &nr);
+    F77_CALL(dgemm)("N", "N", &nr, &nc, &nc, &one, dad, &nr, P, &nc, &zero, res, &nr FCONE FCONE);
     for(int j=0; j<(nc * nr); j++) child[j]*=res[j];
 }
 
@@ -469,7 +450,7 @@ void ExtractScale(int ch, int k, int *nr, int *ntips, double *res){
 // in getDad
 // dad / child * P
 void helpDAD(double *dad, double *child, double *P, int nr, int nc, double *res){
-    F77_CALL(dgemm)(transa, transb, &nr, &nc, &nc, &one, child, &nr, P, &nc, &zero, res, &nr);
+    F77_CALL(dgemm)("N", "N", &nr, &nc, &nc, &one, child, &nr, P, &nc, &zero, res, &nr FCONE FCONE);
     for(int j=0; j<(nc * nr); j++) res[j]=dad[j]/res[j];
 }
 
@@ -505,24 +486,10 @@ SEXP getPrep(SEXP dad, SEXP child, SEXP eve, SEXP evi, SEXP nr, SEXP nc){
         }
     UNPROTECT(1); //RESULT
     return(RESULT);
-    }
-
-
-void NR55(double *eva, int nc, double el, double *w, double *g, SEXP X, int ld, int nr, double *f, double *res){
-    int i, j, k;
-    double *tmp;
-    tmp = (double *) R_alloc(nc, sizeof(double));
-    for(k=0; k<nr; k++) res[k] = 0.0;
-    for(j=0;j<ld;j++){
-//       for(i=0; i<nc ;i++) tmp[i] = w[j] * (eva[i] * g[j]  * el) * exp(eva[i] * g[j] * el);
-        for(i=0; i<nc ;i++) tmp[i] = (eva[i] * g[j]  * el) * exp(eva[i] * g[j] * el);
-        F77_CALL(dgemv)(transa, &nr, &nc, &w[j], REAL(VECTOR_ELT(X, j)), &nr, tmp, &ONE, &one, res, &ONE);
-        }
-    for(i=0; i<nr ;i++) res[i]/=f[i];
 }
 
 
-void NR555(double *eva, int nc, double el, double *w, double *g, SEXP X, int ld, int nr, double *f, double *res){
+void NR_d2f(double *eva, int nc, double el, double *w, double *g, double *X, int ld, int nr, double *f, double *res){
     int i, j, k;
     double *tmp;
     tmp = (double *) R_alloc(nc, sizeof(double));
@@ -531,27 +498,13 @@ void NR555(double *eva, int nc, double el, double *w, double *g, SEXP X, int ld,
 
     for(j=0;j<ld;j++){
         for(i=0; i<nc ;i++) tmp[i] = (eva[i] * g[j]) * exp(eva[i] * g[j] * el);
-        F77_CALL(dgemv)(transa, &nr, &nc, &w[j], REAL(VECTOR_ELT(X, j)), &nr, tmp, &ONE, &one, res, &ONE);
+        F77_CALL(dgemv)("N", &nr, &nc, &w[j], &X[j*(nr*nc)], &nr, tmp, &ONE, &one, res, &ONE FCONE);
         }
     for(i=0; i<nr ;i++) res[i]/=f[i];
 }
 
-// tmp ausserhalb
-void NR66(double *eva, int nc, double el, double *w, double *g, SEXP X, int ld, int nr, double *res){
-    int i, j;
-    double *tmp; //*res,  *dF,
 
-    tmp = (double *) R_alloc(nc, sizeof(double));
-
-    for(j=0;j<ld;j++){
-        for(i=0; i<nc ;i++) tmp[i] = exp(eva[i] * g[j] * el);
-        // alpha = w[j]
-        F77_CALL(dgemv)(transa, &nr, &nc, &w[j], REAL(VECTOR_ELT(X, j)), &nr, tmp, &ONE, &one, res, &ONE);
-    }
-}
-
-
-void NR77(double *eva, int nc, double el, double *w, double *g, double *X, int ld, int nr, double *f, double *res){
+void NR_df(double *eva, int nc, double el, double *w, double *g, double *X, int ld, int nr, double *f, double *res){
     int i, j, k;
     double *tmp;
     tmp = (double *) R_alloc(nc, sizeof(double));
@@ -559,21 +512,21 @@ void NR77(double *eva, int nc, double el, double *w, double *g, double *X, int l
     int nrc = (nc+1L) * nr;
     for(j=0;j<ld;j++){
         for(i=0; i<nc ;i++) tmp[i] = (eva[i] * g[j]  * el) * exp(eva[i] * g[j] * el);
-        F77_CALL(dgemv)(transa, &nr, &nc, &w[j], &X[j*nrc], &nr, tmp, &ONE, &one, res, &ONE);
+        F77_CALL(dgemv)("N", &nr, &nc, &w[j], &X[j*nrc], &nr, tmp, &ONE, &one, res, &ONE FCONE);
         }
     for(i=0; i<nr ;i++) res[i]/=f[i];
 }
 
 
 // tmp ausserhalb?
-void NR88(double *eva, int nc, double el, double *w, double *g, double *X, int ld, int nr, double *res){
+void NR_f(double *eva, int nc, double el, double *w, double *g, double *X, int ld, int nr, double *res){
     int i, j;
     double *tmp; //*res,  *dF,
     tmp = (double *) R_alloc(nc, sizeof(double));
     for(j=0;j<ld;j++){
         for(i=0; i<nc ;i++) tmp[i] = exp(eva[i] * g[j] * el);
         // alpha = w[j]
-        F77_CALL(dgemv)(transa, &nr, &nc, &w[j], &X[j*nr*nc], &nr, tmp, &ONE, &one, res, &ONE);
+        F77_CALL(dgemv)("N", &nr, &nc, &w[j], &X[j*nr*nc], &nr, tmp, &ONE, &one, res, &ONE FCONE);
     }
 }
 
@@ -603,15 +556,15 @@ SEXP LogLik2(SEXP dlist, SEXP P, SEXP nr, SEXP nc, SEXP node, SEXP edge, SEXP nT
             if(ei < nt)
                matp(INTEGER(VECTOR_ELT(dlist, ei)), REAL(contrast), REAL(VECTOR_ELT(P, i)), INTEGER(nr), INTEGER(nc), INTEGER(nco), res);
             else
-               F77_CALL(dgemm)(transa, transb, &nrx, &ncx, &ncx, &one, REAL(VECTOR_ELT(ans, ei-nt)), &nrx,
-                   REAL(VECTOR_ELT(P, i)), &ncx, &zero, res, &nrx);
+               F77_CALL(dgemm)("N", "N", &nrx, &ncx, &ncx, &one, REAL(VECTOR_ELT(ans, ei-nt)), &nrx,
+                   REAL(VECTOR_ELT(P, i)), &ncx, &zero, res, &nrx FCONE FCONE);
             }
         else {
             if(ei < nt)
                 matp(INTEGER(VECTOR_ELT(dlist, ei)), REAL(contrast), REAL(VECTOR_ELT(P, i)), INTEGER(nr), INTEGER(nc), INTEGER(nco), rtmp);
             else
-                F77_CALL(dgemm)(transa, transb, &nrx, &ncx, &ncx, &one, REAL(VECTOR_ELT(ans, ei-nt)), &nrx,
-                    REAL(VECTOR_ELT(P, i)), &ncx, &zero, rtmp, &nrx);
+                F77_CALL(dgemm)("N", "N", &nrx, &ncx, &ncx, &one, REAL(VECTOR_ELT(ans, ei-nt)), &nrx,
+                    REAL(VECTOR_ELT(P, i)), &ncx, &zero, rtmp, &nrx FCONE FCONE);
             for(j=0; j < (nrx*ncx); j++) res[j] *= rtmp[j];
         }
     }
@@ -621,10 +574,10 @@ SEXP LogLik2(SEXP dlist, SEXP P, SEXP nr, SEXP nc, SEXP node, SEXP edge, SEXP nT
 }
 
 // raus
-void matprod(double *x, int nrx, int ncx, double *y, int nry, int ncy, double *z)
-{
-    F77_CALL(dgemm)(transa, transb, &nrx, &ncy, &ncx, &one, x, &nrx, y, &nry, &zero, z, &nrx);
-}
+//void matprod(double *x, int nrx, int ncx, double *y, int nry, int ncy, double *z)
+//{
+//    F77_CALL(dgemm)("N", "N", &nrx, &ncy, &ncx, &one, x, &nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
+//}
 
 
 SEXP getM3(SEXP dad, SEXP child, SEXP P, SEXP nr, SEXP nc){
@@ -636,7 +589,8 @@ SEXP getM3(SEXP dad, SEXP child, SEXP P, SEXP nr, SEXP nc){
     for(i=0; i<n; i++){
         PROTECT(TMP = allocMatrix(REALSXP, nrx, ncx));
         tmp = REAL(TMP);
-        matprod(REAL(VECTOR_ELT(child, i)), nrx, ncx, REAL(VECTOR_ELT(P, i)), ncx, ncx, tmp);
+//        matprod(REAL(VECTOR_ELT(child, i)), nrx, ncx, REAL(VECTOR_ELT(P, i)), ncx, ncx, tmp);
+        F77_CALL(dgemm)("N", "N", &nrx, &ncx, &ncx, &one, REAL(VECTOR_ELT(child, i)), &nrx, REAL(VECTOR_ELT(P, i)), &ncx, &zero, tmp, &nrx FCONE FCONE);
         daddy = REAL(VECTOR_ELT(dad, i));
         for(j=0; j<(ncx * nrx); j++) tmp[j]*=daddy[j];
         SET_VECTOR_ELT(RESULT, i, TMP);
@@ -646,160 +600,34 @@ SEXP getM3(SEXP dad, SEXP child, SEXP P, SEXP nr, SEXP nc){
     return(RESULT);
     }
 
-// SEXP basefreq,
-SEXP FS4(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP X, SEXP dad, SEXP child, SEXP ld, SEXP nr,
-         SEXP weight, SEXP f0, SEXP retA, SEXP retB)
-{
-    SEXP RESULT, EL, P;
-    double *tmp, *f, *wgt=REAL(weight), edle, ledle, newedle, eps=10, *eva=REAL(VECTOR_ELT(eig,0));
-    double ll, lll, delta=0.0, scalep = 1.0, *ws=REAL(w), *gs=REAL(g), l1=0.0, l0=0.0;
-    double y;
-    int i, k=0, ncx=INTEGER(nc)[0], nrx=INTEGER(nr)[0];
-    tmp = (double *) R_alloc(nrx, sizeof(double));
-    f = (double *) R_alloc(nrx, sizeof(double));
-    PROTECT(RESULT = allocVector(VECSXP, 4));
-    edle = REAL(el)[0];
-
-    for(i=0; i<nrx; i++)f[i] = REAL(f0)[i];
-    NR66(eva, ncx, edle, ws, gs, X, INTEGER(ld)[0], nrx, f); // ncx-1L !!!
-    for(i=0; i<nrx ;i++) l0 += wgt[i] * log(f[i]);
-
-    while ( (eps > 1e-05) &&  (k < 5) ) {
-        if(scalep>0.6){
-            NR55(eva, ncx-1L, edle, ws, gs, X, INTEGER(ld)[0], nrx, f, tmp);
-            ll=0.0;
-            lll=0.0;
-            for(i=0; i<nrx ;i++){
-                y = wgt[i]*tmp[i];
-                ll+=y;
-                lll+=y*tmp[i];
-            }
-            delta = ((ll/lll) < 3) ? (ll/lll) : 3;
-        } // end if
-        ledle = log(edle) + scalep * delta;
-        newedle = exp(ledle);
-// some error handling avoid too big small edges & too big steps
-        if (newedle > 10.0) newedle = 10.0;
-        if (newedle < 1e-8) newedle = edle/2;
-        if (newedle < 1e-8) newedle = 1e-8; // 1e-8 phyML
-
-        for(i=0; i<nrx; i++)f[i] = REAL(f0)[i];
-        NR66(eva, ncx, newedle, ws, gs, X, INTEGER(ld)[0], nrx, f);
-        l1 = 0.0;
-        for(i=0; i<nrx ;i++) l1 += wgt[i] * log(f[i]); // + log
-        eps = l1 - l0;
-// some error handling
-        if (eps < 0 || ISNAN(eps)) {
-            if (ISNAN(eps))eps = 0;
-            else {
-                scalep = scalep/2.0;
-                eps = 1.0;
-            }
-            newedle = edle;
-            l1 = l0;
-        }
-        else scalep = 1.0;
-        edle=newedle;
-        l0 = l1;
-        k ++;
-    }
-    PROTECT(EL = ScalarReal(edle));
-    PROTECT(P = getPM(eig, nc, EL, g));
-    SET_VECTOR_ELT(RESULT, 0, EL);
-    if(INTEGER(retA)[0]>0L)SET_VECTOR_ELT(RESULT, 1, getM3(child, dad, P, nr, nc));
-    if(INTEGER(retB)[0]>0L)SET_VECTOR_ELT(RESULT, 2, getM3(dad, child, P, nr, nc));
-// add variance ??
-    SET_VECTOR_ELT(RESULT, 3, ScalarReal(l1));
-    UNPROTECT(3);
-    return (RESULT);
-}
-
-//, SEXP basefreq
-SEXP FS5(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP X, SEXP ld, SEXP nr, SEXP weight, SEXP f0)
-{
-    SEXP RESULT; // EL, P;
-    double *tmp, *f, *wgt=REAL(weight), edle, ledle, newedle, eps=10, *eva=REAL(VECTOR_ELT(eig,0));
-    double ll, lll, delta=0.0, scalep = 1.0, *ws=REAL(w), *gs=REAL(g), l1=0.0, l0=0.0;
-    double y;
-    int i, k=0, ncx=INTEGER(nc)[0], nrx=INTEGER(nr)[0];
-    tmp = (double *) R_alloc(nrx, sizeof(double));
-    f = (double *) R_alloc(nrx, sizeof(double));
-    PROTECT(RESULT = allocVector(REALSXP, 3));
-    edle = REAL(el)[0];
-
-    for(i=0; i<nrx; i++)f[i] = REAL(f0)[i]; //memcpy
-    NR66(eva, ncx, edle, ws, gs, X, INTEGER(ld)[0], nrx, f); // ncx-1L !!!
-    for(i=0; i<nrx ;i++) l0 += wgt[i] * log(f[i]);
-
-    while ( (eps > 1e-05) &&  (k < 10) ) {
-        if(scalep>0.6){
-            NR55(eva, ncx-1L, edle, ws, gs, X, INTEGER(ld)[0], nrx, f, tmp);
-            ll=0.0;
-            lll=0.0;
-            for(i=0; i<nrx ;i++){
-                y = wgt[i]*tmp[i];
-                ll+=y;
-                lll+=y*tmp[i];
-            }
-            delta = ((ll/lll) < 3) ? (ll/lll) : 3;
-        } // end if
-        ledle = log(edle) + scalep * delta;
-        newedle = exp(ledle);
-// some error handling avoid too big small edges & too big steps
-        if (newedle > 10.0) newedle = 10.0;
-//        if (newedle < 1e-8) newedle = edle/2;
-        if (newedle < 1e-8) newedle = 1e-8; // 1e-8 phyML
-
-        for(i=0; i<nrx; i++)f[i] = REAL(f0)[i];
-        NR66(eva, ncx, newedle, ws, gs, X, INTEGER(ld)[0], nrx, f);
-        l1 = 0.0;
-        for(i=0; i<nrx ;i++) l1 += wgt[i] * log(f[i]); // + log
-        eps = l1 - l0;
-// some error handling
-        if (eps < 0 || ISNAN(eps)) {
-            if (ISNAN(eps))eps = 0;
-            else {
-                scalep = scalep/2.0;
-                eps = 1.0;
-            }
-            newedle = edle;
-            l1 = l0;
-        }
-        else scalep = 1.0;
-        edle=newedle;
-        l0 = l1;
-        k ++;
-    }
-// variance
-    NR555(eva, ncx-1L, edle, ws, gs, X, INTEGER(ld)[0], nrx, f, tmp);
-    lll=0.0;
-    for(i=0; i<nrx ;i++) lll+=wgt[i]*tmp[i]*tmp[i];
-    REAL(RESULT)[0] = edle;
-    REAL(RESULT)[1] = 1.0 / lll;  // variance
-    REAL(RESULT)[2] = l0; //l0
-    UNPROTECT(1);
-    return (RESULT);
-}
 
 
 void fs3(double *eva, int nc, double el, double *w, double *g, double *X, int ld, int nr, double *weight,
-   double *f0, double *res)
+         double *f0, double tau, double *res) //, int mkv)
 {
+    int mkv=0;
     double *tmp, *f, edle, ledle, newedle, eps=10;
     double ll=0.0, lll, delta=0.0, scalep = 1.0, l1=0.0, l0=0.0;
-    double y;
-    int i, k=0;
+    double y, p0=0.0; //sum_wgt=0.0,
+    int i, j, k=0;
     tmp = (double *) R_alloc(nr, sizeof(double));
     f = (double *) R_alloc(nr, sizeof(double));
     edle = el; //REAL(el)[0];
 
     for(i=0; i<nr; i++)f[i] = f0[i];
-    NR88(eva, nc, edle, w, g, X, ld, nr, f); // nc-1L !!!
-    for(i=0; i<nr ;i++) l0 += weight[i] * log(f[i]);
+    NR_f(eva, nc, edle, w, g, X, ld, nr, f); // nc-1L !!!
+    if(mkv){
+      for(j=0; j<nc; j++)p0 += f[i];
+      for(i=0; i<nr ;i++) f[i] /= (1-p0);
+    }
+    for(i=0; i<nr ;i++){
+      l0 += weight[i] * log(f[i]);
+      //sum_wgt += weight[i];
+    }
 
     while ( (eps > 1e-05) &&  (k < 10) ) {
         if(scalep>0.6){
-            NR77(eva, nc-1L, edle, w, g, X, ld, nr, f, tmp);
+            NR_df(eva, nc-1L, edle, w, g, X, ld, nr, f, tmp);
             ll=0.0;
             lll=0.0;
             for(i=0; i<nr ;i++){
@@ -811,16 +639,20 @@ void fs3(double *eva, int nc, double el, double *w, double *g, double *X, int ld
         } // end if
         ledle = log(edle) + scalep * delta;
         newedle = exp(ledle);
-// some error handling avoid too big small edges & too big steps
+        // some error handling avoid too big small edges & too big steps
         if (newedle > 10.0) newedle = 10.0;
-        if (newedle < 1e-8) newedle = 1e-8; // 1e-8 phyML
+        if (newedle < tau) newedle = tau; // 1e-8 phyML
 
         for(i=0; i<nr; i++)f[i] = f0[i];
-        NR88(eva, nc, newedle, w, g, X, ld, nr, f);
+        NR_f(eva, nc, newedle, w, g, X, ld, nr, f);
+        if(mkv){
+          for(j=0; j<nc; j++)p0 += f[i];
+          for(i=0; i<nr ;i++) f[i] /= (1-p0);
+        }
         l1 = 0.0;
         for(i=0; i<nr ;i++) l1 += weight[i] * log(f[i]);
         eps = l1 - l0;
-// some error handling
+        // some error handling
         if (eps < 0 || ISNAN(eps)) {
             if (ISNAN(eps))eps = 0;
             else {
@@ -835,26 +667,62 @@ void fs3(double *eva, int nc, double el, double *w, double *g, double *X, int ld
         l0 = l1;
         k ++;
     }
-// variance n
-//   NR555(eva, ncx-1L, edle, ws, gs, X, INTEGER(ld)[0], nrx, f, tmp);
-//   lll=0.0;
-//   for(i=0; i<nrx ;i++) lll+=wgt[i]*tmp[i]*tmp[i];
+    // variance n
+    NR_d2f(eva, nc-1L, edle, w, g, X, ld, nr, f, tmp);
+    lll=0.0;
+    for(i=0; i<nr ;i++) lll+=weight[i]*tmp[i]*tmp[i];
     res[0] = edle;
-    res[1] = ll;
+    res[1] = 1.0 / lll;
     res[2] = l0; //l0
-// return res[3] = 1 or 0 for success
+    // return res[3] = 1 or 0 for success
+}
+
+
+// in fs()
+SEXP FS4(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP X, SEXP dad, SEXP child, SEXP ld, SEXP nr,
+         SEXP weight, SEXP f0, SEXP tau, SEXP retA, SEXP retB)
+{
+    SEXP RESULT, EL, P;
+    double *tmp, *wgt=REAL(weight), edle, *eva=REAL(VECTOR_ELT(eig,0));
+    int ncx=INTEGER(nc)[0], nrx=INTEGER(nr)[0];
+    tmp = (double *) R_alloc(3L, sizeof(double));
+    PROTECT(RESULT = allocVector(VECSXP, 4));
+    edle = REAL(el)[0];
+    fs3(eva, ncx, edle, REAL(w), REAL(g), REAL(X), INTEGER(ld)[0], nrx, wgt, REAL(f0), REAL(tau)[0], tmp);
+    PROTECT(EL = ScalarReal(tmp[0]));
+    PROTECT(P = getPM(eig, nc, EL, g));
+    SET_VECTOR_ELT(RESULT, 0, EL);
+    if(INTEGER(retA)[0]>0L)SET_VECTOR_ELT(RESULT, 1, getM3(child, dad, P, nr, nc));
+    if(INTEGER(retB)[0]>0L)SET_VECTOR_ELT(RESULT, 2, getM3(dad, child, P, nr, nc));
+// add variance ??
+    SET_VECTOR_ELT(RESULT, 3, ScalarReal(tmp[2]));
+    UNPROTECT(3);
+    return (RESULT);
+}
+
+// in dist.ml
+SEXP FS5(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP X, SEXP ld, SEXP nr, SEXP weight, SEXP f0, SEXP tau)
+{
+    SEXP RESULT; // EL, P;
+    double *wgt=REAL(weight), *eva=REAL(VECTOR_ELT(eig,0));
+    int ncx=INTEGER(nc)[0], nrx=INTEGER(nr)[0];
+    PROTECT(RESULT = allocVector(REALSXP, 3));
+    double edle = REAL(el)[0];
+    fs3(eva, ncx, edle, REAL(w), REAL(g), REAL(X), INTEGER(ld)[0], nrx, wgt, REAL(f0), REAL(tau)[0], REAL(RESULT));
+    UNPROTECT(1);
+    return (RESULT);
 }
 
 
 SEXP optE(SEXP PARENT, SEXP CHILD, SEXP ANC, SEXP eig, SEXP EVI, SEXP EL,
                   SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP NTIPS, SEXP CONTRAST,
                   SEXP CONTRAST2, SEXP NCO,
-                  SEXP dlist, SEXP WEIGHT, SEXP F0){
+                  SEXP dlist, SEXP WEIGHT, SEXP F0, SEXP TAU){
     int i, k=length(W), h, j, n=length(PARENT), m, lEL=length(EL);
     int nc=INTEGER(NC)[0], nr=INTEGER(NR)[0], ntips=INTEGER(NTIPS)[0];
-    int *parent=INTEGER(PARENT), *child=INTEGER(CHILD), *anc=INTEGER(ANC); //
-    int loli, nco =INTEGER(NCO)[0]; //=INTEGER(LOLI)[0]
-    double *weight=REAL(WEIGHT), *f0=REAL(F0), *w=REAL(W);
+    int *parent=INTEGER(PARENT), *child=INTEGER(CHILD), *anc=INTEGER(ANC);
+    int loli, nco =INTEGER(NCO)[0];
+    double *weight=REAL(WEIGHT), *f0=REAL(F0), *w=REAL(W), tau=REAL(TAU)[0];
     double *g=REAL(G), *evi=REAL(EVI), *contrast=REAL(CONTRAST), *contrast2=REAL(CONTRAST2);
     double *el; //=REAL(EL);
     double *eva, *eve, *evei, *tmp, *P;
@@ -917,7 +785,7 @@ SEXP optE(SEXP PARENT, SEXP CHILD, SEXP ANC, SEXP eig, SEXP EVI, SEXP EL,
             }
         }
     }
-    fs3(eva, nc, oldel, w, g, X, k, nr, weight, f0, res);
+    fs3(eva, nc, oldel, w, g, X, k, nr, weight, f0, tau, res);
     updateLL2(dlist, pa, ch, eva, eve, evei, res[0], g, nr,
         nc, ntips, contrast, nco, k, tmp, P);
         el[ch-1L] = res[0];
@@ -932,12 +800,12 @@ SEXP optE(SEXP PARENT, SEXP CHILD, SEXP ANC, SEXP eig, SEXP EVI, SEXP EL,
 SEXP optQrtt(SEXP PARENT, SEXP CHILD, SEXP eig, SEXP EVI, SEXP EL,
           SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP NTIPS, SEXP CONTRAST,
           SEXP CONTRAST2, SEXP NCO,
-          SEXP dlist, SEXP WEIGHT, SEXP F0){
+          SEXP dlist, SEXP WEIGHT, SEXP F0, SEXP TAU){
     int i, k=length(W), h, j, m, lEL=length(EL);
     int nc=INTEGER(NC)[0], nr=INTEGER(NR)[0], ntips=INTEGER(NTIPS)[0];
     int *parent=INTEGER(PARENT), *child=INTEGER(CHILD), pa, ch;
     int nco =INTEGER(NCO)[0]; // loli,
-    double *weight=REAL(WEIGHT), *f0=REAL(F0), *w=REAL(W);
+    double *weight=REAL(WEIGHT), *f0=REAL(F0), *w=REAL(W), tau=REAL(TAU)[0];
     double *g=REAL(G), *evi=REAL(EVI), *contrast=REAL(CONTRAST), *contrast2=REAL(CONTRAST2);
     double *el, *X;
     double *eva, *eve, *evei, *tmp, *P;
@@ -987,7 +855,7 @@ SEXP optQrtt(SEXP PARENT, SEXP CHILD, SEXP eig, SEXP EVI, SEXP EL,
                 }
             }
         }
-        fs3(eva, nc, oldel, w, g, X, k, nr, weight, f0, res);
+        fs3(eva, nc, oldel, w, g, X, k, nr, weight, f0, tau, res);
 // go up
 // if i=2 go down
         if(m==2)updateLLQ(dlist, ch, pa, eva, eve, evei, res[0], g, nr,
@@ -999,39 +867,3 @@ SEXP optQrtt(SEXP PARENT, SEXP CHILD, SEXP eig, SEXP EVI, SEXP EL,
     UNPROTECT(1); //RESULT
     return(RESULT);
 }
-
-
-
-SEXP PML4(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP nco, SEXP contrast, SEXP N){
-    int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, j, indLL;
-    int nTips = INTEGER(NTips)[0], *SC, *sc;
-    double *g=REAL(G), *w=REAL(W), *tmp, *res;
-    SEXP TMP;
-    double *eva, *eve, *evei;
-    eva = REAL(VECTOR_ELT(eig, 0));
-    eve = REAL(VECTOR_ELT(eig, 1));
-    evei = REAL(VECTOR_ELT(eig, 2));
-    SC = (int *) R_alloc(nr * k, sizeof(int));
-    sc = (int *) R_alloc(nr, sizeof(int));
-    tmp = (double *) R_alloc(nr * k, sizeof(double));
-    PROTECT(TMP = allocVector(REALSXP, nr));
-
-    res=REAL(TMP);
-    for(i=0; i<(k*nr); i++)tmp[i]=0.0;
-    indLL = nr * nc * nTips;
-    for(i=0; i<k; i++){
-        lll3(dlist, eva, eve, evei, REAL(EL), g[i], &nr, &nc, INTEGER(node), INTEGER(edge), nTips, REAL(contrast), INTEGER(nco)[0], INTEGER(N)[0],  &SC[nr * i], REAL(bf), &tmp[i*nr], &LL[indLL *i], &SCM[nr * nTips * i]);
-    }
-    rowMinScale(SC, nr, k, sc);
-    for(i=0; i<nr; i++){
-        res[i]=0.0;
-        for(j=0;j<k;j++)res[i] += w[j] * exp(LOG_SCALE_EPS * SC[i+j*nr]) * tmp[i+j*nr];
-    }
-    for(i=0; i<nr; i++) res[i] = log(res[i]) + LOG_SCALE_EPS * sc[i];
-    UNPROTECT(1);
-    return TMP;
-}
-
-
-
-
